@@ -1,18 +1,20 @@
 import gzip
 import bz2
 from collections import defaultdict
-from reademptionlib.fasta import FastaParser
-from reademptionlib.fastq import FastqParser
 from reademptionlib.polyaclipper import PolyAClipper
+from Bio import SeqIO
 
 class ReadProcessor(object):
 
     def __init__(self, poly_a_clipping=False,  min_read_length=12, 
-                 paired_end=False, fastq=False):
+                 paired_end=False, fastq=False, min_phred_score=None,
+                 adapter=None):
         self._poly_a_clipping = poly_a_clipping
         self._min_read_length = min_read_length
         self._paired_end = paired_end
         self._fastq = fastq
+        self._min_phred_score = min_phred_score
+        self._adapter = adapter
         self._poly_a_clipper = PolyAClipper()
 
     def process_single_end(self, input_path, output_path):
@@ -49,75 +51,97 @@ class ReadProcessor(object):
         Can deal with plain fasta files, gzipped fasta or bzipped2 fasta.
         """
         if input_path.endswith(".gz"):
-            return self._line_interator(input_path, gzip.open)
+            return gzip.open(input_path, "rt")
         elif input_path.endswith(".bz2"):
-            return self._line_interator(input_path, bz2.open)
+            return bz2.open(input_path, "rt")
         return open(input_path)
 
-    def _line_interator(self, input_path, open_func):
-        """Return a line iterator that decodes the bytes"""
-        with open_func(input_path, "rb") as fh:
-            for line in fh:
-                yield(line.decode())
+    def _trim_by_quality(self, seq, qualities):
+        good_nucl = []
+        for nucl, qual in zip(seq, qualities):
+            if qual < self._min_phred_score:
+                break
+            good_nucl.append(nucl)
+        return "".join(good_nucl)
 
+    def _clip_adapter(self, seq):
+        adapter_start_pos = seq.find(self._adapter)
+        if adapter_start_pos == -1:
+            return seq
+        else:
+            return seq[:adapter_start_pos]
+        
     def _process_single_end(self, input_fh, output_fh):
-        for header, seq in self._seq_parser().entries(input_fh):
+        for header, seq, qualities in self._parse_sequences(input_fh):
+            raw_seq_len = len(seq)
             self._stats["total_no_of_reads"] += 1
+            if self._fastq and not self._min_phred_score is None:
+                seq = self._trim_by_quality(seq, qualities)
+            if not self._adapter is None:
+                seq = self._clip_adapter(seq)
             if self._poly_a_clipping:
-                clipped_seq = self._poly_a_clipper.clip_poly_a_strech(seq)
-                clipped_seq = self._poly_a_clipper.remove_3_prime_a(clipped_seq)
-            else:
-                clipped_seq = seq
-            if len(clipped_seq) == len(seq) - 1:
+                seq = self._poly_a_clipper.clip_poly_a_strech(seq)
+                seq = self._poly_a_clipper.remove_3_prime_a(seq)
+            clipped_seq_len = len(seq)
+            if clipped_seq_len == raw_seq_len - 1:
                 self._stats["single_a_removed"] += 1
-            elif len(clipped_seq) < len(seq) - 1:
+            elif clipped_seq_len < raw_seq_len - 1:
                 self._stats["polya_removed"] += 1
             else:
                 self._stats["unmodified"] += 1
-            clipped_seq_len = len(clipped_seq)
             if clipped_seq_len < self._min_read_length:
                 self._stats["too_short"] += 1
                 continue
-            self._stats["long_enough"] += 1
-            raw_seq_len = len(seq)
             self._stats["read_length_before_processing_and_freq"][
                 raw_seq_len] += 1
             self._stats["read_length_after_processing_and_freq"][
                 clipped_seq_len] += 1
             # Encoding to bytes is necessary due to saving via gzip
-            output_fh.write(str.encode(">%s\n%s\n" % (header, clipped_seq)))
+            output_fh.write(str.encode(">%s\n%s\n" % (header, seq)))
 
+    def _parse_sequences(self, input_fh):
+        if self._fastq:
+            for seq_record in SeqIO.parse(input_fh, "fastq"):
+                yield(seq_record.description, str(seq_record.seq), 
+                      seq_record.letter_annotations["phred_quality"])
+        else:
+            for seq_record in SeqIO.parse(input_fh, "fasta"):
+                yield(seq_record.description, str(seq_record.seq), None)
+        
     def _process_paired_end(
         self, input_p1_fh, input_p2_fh, output_p1_fh, output_p2_fh):
         for fasta_entry_p1, fasta_entry_p2 in zip(
-            self._seq_parser().entries(input_p1_fh), 
-            self._seq_parser().entries(input_p2_fh)):
+                self._parse_sequences(input_p1_fh),
+                self._parse_sequences(input_p2_fh,)):
             header_p1 = fasta_entry_p1[0]
             header_p2 = fasta_entry_p2[0]
             seq_p1 = fasta_entry_p1[1]
             seq_p2 = fasta_entry_p2[1]
+            qualities_p1 = fasta_entry_p1[2]
+            qualities_p2 = fasta_entry_p1[2]
+            raw_seq_p1_len = len(seq_p1)
+            raw_seq_p2_len = len(seq_p2)
             self._stats["total_no_of_reads"] += 1
             self._stats["unmodified"] += 1
-            seq_p1_len = len(seq_p1)
-            seq_p2_len = len(seq_p2)
-            if (seq_p1_len < self._min_read_length or 
-                seq_p2_len < self._min_read_length):
+            if self._fastq and not self._min_phred_score is None:
+                seq_p1 = self._trim_by_quality(seq_p1, qualities_p1)
+                seq_p2 = self._trim_by_quality(seq_p2, qualities_p2)
+            if not self._adapter is None:
+                seq_p1 = self._clip_adapter(seq_p1)
+                seq_p2 = self._clip_adapter(seq_p2)
+            if (raw_seq_p1_len < self._min_read_length or 
+                raw_seq_p2_len < self._min_read_length):
                 self._stats["too_short"] += 1
                 continue
             self._stats["long_enough"] += 1
             self._stats["read_length_before_processing_and_freq"][
-                seq_p1_len] += 1
+                raw_seq_p1_len] += 1
             self._stats["read_length_after_processing_and_freq"][
-                seq_p1_len] += 1
+                raw_seq_p1_len] += 1
             self._stats["read_length_before_processing_and_freq"][
-                seq_p2_len] += 1
+                raw_seq_p2_len] += 1
             self._stats["read_length_after_processing_and_freq"][
-                seq_p2_len] += 1
+                raw_seq_p2_len] += 1
             # Encoding to bytes is necessary due to saving via gzip
             output_p1_fh.write(str.encode(">%s\n%s\n" % (header_p1, seq_p1)))
             output_p2_fh.write(str.encode(">%s\n%s\n" % (header_p2, seq_p2)))
-
-    def _seq_parser(self):
-        if self._fastq: return FastqParser()
-        else: return FastaParser()
-
